#include "x509.h"

#include "asndata.def"

// I define my own policiy identifier with 2 qualifiers (mere examples)
// This definition will allow understanding of policy qualifier in cert extension

ObjectIdentifier myPol( 1, 2, 65536 ) ;  // mere example ids
ObjectIdentifier low( myPol, 1 ) ;
ObjectIdentifier level( myPol, 2 ) ;


void main()
{
	// initialize policy qualifiers
	SupportedPolicyQualifiers.IdentifiedBy( low, new Notype ) ;
	SupportedPolicyQualifiers.IdentifiedBy( level, new Integer ) ;

	// initialize other names with a "postal name"
	SupportedOtherNames.IdentifiedBy( id_at_postalAddress, new PostalAddress ) ;

	ofile os( "certv3.ber" ) ;

	Certificate c ;
	c.cert().version().set( Version::v3 ) ;
	c.cert().serialNumber().set( 256 ) ;
	c.cert().signature().set( md2WithRSAEncryption ) ;
	Name nm ;
	nm.set( id_at_countryName, PrintableString( "ES" ) ) ;
	nm.set( id_at_organization, PrintableString( "Universitat Politecnica de Catalunya" ) ) ;
	nm.set( id_at_organizationalUnit, PrintableString( "Dept. Arquitectura de Computadors" ) ) ;
	c.cert().issuer().set( nm ) ;
	c.cert().validity().notBefore().set( time(0) ) ;
	c.cert().validity().notAfter().set( Asn1Time::offset( time(0), 1 ) ) ;
	nm.set( id_at_commonName, PrintableString( "Francisco Jordan" ) ) ;
	c.cert().subject().set( nm ) ;
	c.cert().subjectPublicKeyInfo().algorithm().set( rsaEncryption ) ;
	c.cert().subjectPublicKeyInfo().subjectPublicKeyBits().set( "'3068026100BEAA8B7754A3AFCA779F2FB0CF4388FFA66D79555B618C68EC481E8A8638A4FE19B862171D9D0F472CFF638F299104D152BC7F67B6B28F7455C133216C8FAB019524C8B273939D226150A935FB9D575032EF56525093ABB18894785615C61C8B0203010001'H" ) ;
	c.signatureAlgorithm().set( md2WithRSAEncryption ) ;
	c.signature().set( "'5C01BDB54188877A0ED30E6B3ABF04EA04CB5F61723CA3BD78F56617FE373AABEB67BFB7DAA838F6331571752FB98C91A0E487BA4B43A0228FD3A9864389E6505C01BDB54188877A0ED30E6B3ABF04EA04CB5F61723CA3BD78F56617FE373AABEB67BFB7DAA838F6331571752FB98C91A0E487BA4B43A0228FD3A9864389E650'H" ) ;

	// extensions

	AuthorityKeyIdentifier ak ;
	ak.keyIdentifier().set( "'0E6B3ABF04EA04C30E6B3ABF04EA04C3'H" ) ;
	c.cert().extensions(0).set( id_ce_authorityKeyIdentifier, ak ) ;

	c.cert().extensions(1).set( id_ce_keyUsage, KeyUsage( KeyUsage::digitalSignature ), TRUE ) ;

	CertificatePoliciesSyntax polS ;
	PolicyInformation pol ;
	PolicyQualifierInfo ele ;
	pol.policyIdentifier().set( myPol ) ;
	ele.policyQualifierId().set( low ) ;	// no qualifier
	pol.policyQualifiers(0).set( ele ) ;
	ele.policyQualifierId().set( level ) ;
	ele.qualifier().set( Integer( 10 ) ) ;	// qualifier
	pol.policyQualifiers(1).set( ele ) ;
	polS.policyInformation(0).set( pol ) ;
	c.cert().extensions(3).set( id_ce_certificatePolicies, polS ) ;

	GeneralNames gn ;
	gn.generalName(0).rfc822Name().set( "jordan@ac.upc.es" ) ;
	gn.generalName(1).uniformResourceIdentifier().set( "http://ac.upc.es/" ) ;
	PostalAddress mypost ;
	mypost.line(0).printableString().set( "Dr. Francisco Jordan" ) ;
	mypost.line(1).printableString().set( "U.P.C" ) ;
	mypost.line(2).printableString().set( "C/ Gran Capita sn, Mod-D6" ) ;
	mypost.line(3).printableString().set( "08071-Barcelona, Spain" ) ;
	gn.generalName(2).otherName().set( id_at_postalAddress, mypost ) ;
	c.cert().extensions(4).set( id_ce_subjectAltName, gn ) ;

	c.cert().extensions(5).set( id_ce_basicConstraints, BasicConstraintsSyntax(), TRUE ) ;

	gn.clear() ;
	gn.generalName(1).uniformResourceIdentifier().set( "http://www.upc.es/" ) ;
	c.cert().extensions(6).set( id_ce_issuerAltName, gn ) ;

	c.encode( os ) ;
}
